cdef extern from "ggwave.h" nogil:

    ctypedef enum ggwave_SampleFormat:
        GGWAVE_SAMPLE_FORMAT_UNDEFINED,
        GGWAVE_SAMPLE_FORMAT_U8,
        GGWAVE_SAMPLE_FORMAT_I8,
        GGWAVE_SAMPLE_FORMAT_U16,
        GGWAVE_SAMPLE_FORMAT_I16,
        GGWAVE_SAMPLE_FORMAT_F32

    ctypedef enum ggwave_ProtocolId:
        GGWAVE_PROTOCOL_AUDIBLE_NORMAL,
        GGWAVE_PROTOCOL_AUDIBLE_FAST,
        GGWAVE_PROTOCOL_AUDIBLE_FASTEST,
        GGWAVE_PROTOCOL_ULTRASOUND_NORMAL,
        GGWAVE_PROTOCOL_ULTRASOUND_FAST,
        GGWAVE_PROTOCOL_ULTRASOUND_FASTEST,
        GGWAVE_PROTOCOL_DT_NORMAL,
        GGWAVE_PROTOCOL_DT_FAST,
        GGWAVE_PROTOCOL_DT_FASTEST,
        GGWAVE_PROTOCOL_MT_NORMAL,
        GGWAVE_PROTOCOL_MT_FAST,
        GGWAVE_PROTOCOL_MT_FASTEST,

        GGWAVE_PROTOCOL_CUSTOM_0,
        GGWAVE_PROTOCOL_CUSTOM_1,
        GGWAVE_PROTOCOL_CUSTOM_2,
        GGWAVE_PROTOCOL_CUSTOM_3,
        GGWAVE_PROTOCOL_CUSTOM_4,
        GGWAVE_PROTOCOL_CUSTOM_5,
        GGWAVE_PROTOCOL_CUSTOM_6,
        GGWAVE_PROTOCOL_CUSTOM_7,
        GGWAVE_PROTOCOL_CUSTOM_8,
        GGWAVE_PROTOCOL_CUSTOM_9

    enum:
        GGWAVE_OPERATING_MODE_RX,
        GGWAVE_OPERATING_MODE_TX,
        GGWAVE_OPERATING_MODE_RX_AND_TX,
        GGWAVE_OPERATING_MODE_TX_ONLY_TONES,
        GGWAVE_OPERATING_MODE_USE_DSS

    ctypedef struct ggwave_Parameters:
        int payloadLength
        float sampleRateInp
        float sampleRateOut
        float sampleRate
        int samplesPerFrame
        float soundMarkerThreshold
        ggwave_SampleFormat sampleFormatInp
        ggwave_SampleFormat sampleFormatOut
        int operatingMode

    ctypedef int ggwave_Instance

    ggwave_Parameters ggwave_getDefaultParameters();

    ggwave_Instance ggwave_init(const ggwave_Parameters parameters);

    void ggwave_free(ggwave_Instance instance);

    int ggwave_encode(
            ggwave_Instance instance,
            const void * payloadBuffer,
            int payloadSize,
            ggwave_ProtocolId protocolId,
            int volume,
            void * waveformBuffer,
            int query);

    int ggwave_decode(
            ggwave_Instance instance,
            const void * waveformBuffer,
            int waveformSize,
            void * payloadBuffer);

    void ggwave_setLogFile(void * fptr);

    void ggwave_rxToggleProtocol(
            ggwave_ProtocolId protocolId,
            int state);

    void ggwave_txToggleProtocol(
            ggwave_ProtocolId protocolId,
            int state);



# cython: language_level=3

from libc.stdio cimport stderr

cimport cggwave

def getDefaultParameters():
    return cggwave.ggwave_getDefaultParameters()

def init(parameters = None):
    if (parameters is None):
        parameters = getDefaultParameters()

    return cggwave.ggwave_init(parameters)

def free(instance):
    return cggwave.ggwave_free(instance)

def encode(payload, protocolId = 1, volume = 10, instance = None):
    """ Encode payload into an audio waveform.
        @param {string} payload, the data to be encoded
        @return Generated audio waveform bytes representing 16-bit signed integer samples.
    """

    cdef bytes data_bytes = payload.encode()
    cdef char* cdata = data_bytes

    own = False
    if (instance is None):
        own = True
        instance = init(getDefaultParameters())

    n = cggwave.ggwave_encode(instance, cdata, len(data_bytes), protocolId, volume, NULL, 1)

    cdef bytes output_bytes = bytes(n)
    cdef char* coutput = output_bytes

    n = cggwave.ggwave_encode(instance, cdata, len(data_bytes), protocolId, volume, coutput, 0)

    if (own):
        free(instance)

    return output_bytes

def decode(instance, waveform):
    """ Analyze and decode audio waveform to obtain original payload
        @param {bytes} waveform, the audio waveform to decode
        @return The decoded payload if successful.
    """

    cdef bytes data_bytes = waveform
    cdef char* cdata = data_bytes

    cdef bytes output_bytes = bytes(256)
    cdef char* coutput = output_bytes

    rxDataLength = cggwave.ggwave_decode(instance, cdata, len(data_bytes), coutput)

    if (rxDataLength > 0):
        return coutput[0:rxDataLength]

    return None

def disableLog():
    cggwave.ggwave_setLogFile(NULL);

def enableLog():
    cggwave.ggwave_setLogFile(stderr);

def rxToggleProtocol(protocolId, state):
    cggwave.ggwave_rxToggleProtocol(protocolId, state);

def txToggleProtocol(protocolId, state):
    cggwave.ggwave_txToggleProtocol(protocolId, state);



from setuptools import setup, Extension
from codecs import open
from Cython.Build import cythonize
import os

long_description = ""

# Load README.rst into long description. User can skip using README.rst as long description:
# GGWAVE_OMIT_README_RST=1 python setup.py install
OMIT_README_RST = os.getenv('GGWAVE_OMIT_README_RST', False)
if not OMIT_README_RST:
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

extension = [Extension("ggwave",
                       ["src/ggwave/ggwave.pyx", "src/ggwave/ggwave/src/ggwave.cpp"],
                       include_dirs=["src/ggwave/ggwave/include", "src/ggwave/ggwave/include/ggwave"],
                       depends=["src/ggwave/ggwave/include/ggwave/ggwave.h"],
                       language="c++",
                       )]

extension = cythonize(extension)

setup(
    # Information
    name="ggwave-wheels",
    description="Tiny data-over-sound library.",
    long_description=long_description,
    url="https://github.com/ggerganov/ggwave",
    author="Georgi Gerganov",
    author_email="ggerganov@gmail.com",
    license="MIT",
    keywords="data-over-sound fsk ecc serverless pairing qrcode ultrasound",
    # Build instructions
    ext_modules=extension,
)
